import uuid

from pymongo import MongoClient
from pymongo import errors


import dvr_logger

class DvrMache(object):

  HOST = "mongo"
  PORT = 27017
  DATABASE = "sd_cache"
  COLLECTION = "data"


  def connect(self):
    dvr_logger.lager.info("Attempting to connect to {} on {}.".format(self.HOST, self.PORT))
    try:
      client = MongoClient(host=self.HOST, port=self.PORT)
    except errors.ConnectionFailure as me:
      raise Exception("Error connecting to Dache.")

    db = client["sd_cache"]
    return db


  def _change_record(self, cmd, data, key, Id):
    db = self.connect()
    col = db["data"]

    try:
      result = col.update_one({"_id": Id},{cmd: {key: data}}, upsert=True)
    except Exception as pe:
      raise Exception("{} / {} {} - ID: {}/LOC: {}.".format(pe, cmd, data, Id, key))

    dvr_logger.lager.info("Updated {} - ID: {}/LOC: {}.".format(data, Id, key))
    return Id


  def add(self, data, key, Id):
    Id = self._change_record("$push", data, key, Id)
    return Id



  def put(self, data, key):
    Id = str(uuid.uuid4())
    Id = self._change_record("$set", data, key, Id)
    return Id


  def get(self, Id):
    db = self.connect()
    try:
      result = db["data"].find_one({"_id": Id})
    except errors.PyMongoError as pe:  
      raise Exception("{} / Finding {}".format(pe, Id))

    dvr_logger.lager.info("Retrieved data with Id = {}".format(Id))
    return result 


  def delete(self):
    db = self.connect()
    dvr_logger.lager.info("Deleting collection.")
    db["data"].drop()
    return True




import logging
import socket
import logstash

host = "logstash"
if host != "logstash":
  # Configured for use with logstash link in docker
  lager = logging.getLogger("Satellite Driver")
  lager.addHandler(logstash.LogstashHandler(host, 5000, version=1))
  lager.setLevel(logging.WARNING)
else:
  # Configure for testing 
  logging.basicConfig()
  lager = logging.getLogger("Testing") 
  lager.setLevel(logging.WARNING)



import json

import requests

from dvr_logger import lager 


class DvrRequester(object):
  HOST = "https://api.buildingos.com/"

  def __init__(self, username, password, client_id, client_secret):
    self.bos_token= ""
    self.params = {
      "username": username,
      "password": password,
      "grant_type": "password",
      "client_secret": client_secret,
      "client_id": client_id
    }


  def _requester(self, url, headers=None, params=None, cookies=None):
    response = requests.get(url, params=params, headers=headers, cookies=cookies)

    try:
      response.raise_for_status()
    except requests.exceptions.HTTPError as e:
      lager.error("{} / on call {}".format(e, url))
      raise Exception("{} / Error making call on {}.".format(e, url))
 
    lager.warning("Recieved resoonse {} on call {}.".format(url, response))
    return response


  def post(self, url, params=None, data=None, headers=None):
    try:
      data = json.dumps(data)
    except ValueError:
      lager.info("No data arg passed.")
      
    response = requests.post(url, params=params, data=data, headers=headers)

    try:
      response.raise_for_status()
    except requests.exceptions.HTTPError as e:
      lager.error("{} / on call {}".format(e, url))
      raise Exception("{} / Error making call on {}.".format(e, url))

    return response
 

  def get_bos_token(self):
    response = self.post(self.HOST + "o/token/", params=self.params)

    try:
      content = response.json()
    except ValueError:
      return None

    self.bos_token = content["access_token"]
    lager.warning("Retrieved BOS token: {}.".format(self.bos_token))
    return self.bos_token


  def get(self, url):
    headers = {"Authorization" : "Bearer " + self.bos_token}
    lager.error("Here is the header {}.".format(headers))

    try:
      response = self._requester(url, headers=headers, params=None, cookies=None)
      return response
    except Exception as e:
      lager.error("{} / Accessing bos API.".format(e))
      self.get_bos_token()
      return self.get(url) 




from setuptools import setup


setup(name="sdlib_v0.8",
  version="0.8",
  description="Easily integrate developer driver with BuildingOS platform.",
  author="Malcolm I. Monroe", 
  author_email="malcolmian.monroe@gmail.com",
  url="https://github.com/mmonroe86/sdlib.git",
  py_modules=["dvr_mache", "dvr_logger", "dvr_requester"],
  install_requires=[
    "pymongo",  
    "python-logstash",
    "requests",
    "celery",
    "flower",
    "uwsgi",  
  ],
  zip_safe=False
)
